#include "cute.h"
#include "ide_listener.h"
#include "cute_runner.h"

#include "calc.hpp"

template <typename T>
std::stack<T>& operator<<(std::stack<T>& s, const T& elem) {
	s.push(elem);
	return s;
}

template <typename T>
std::stack<T>& operator>>(std::stack<T>& s, T& elem) {
	elem = s.top();
	s.pop();
	return s;
}

template <typename T>
std::stack<T>& operator<<(std::stack<T>& s, const std::string& elem) {
	//ignore strings for now, maybe handling of operands ?!?
	return s;
}

struct CalcTest {

private:
	std::stack<int> ops;
	Calculator calc;

public :
  CalcTest() : calc(&ops) {}

  void testReq1Addition() {
	  ops << 6 << 9;
	  calc.compute(PLUS);
	  ASSERT_EQUAL(1, ops.size());
	  ASSERT_EQUAL(15, ops.top());
  };

  void testReq1AdditionZero(){
	  ops << 0 << 0;
	  calc.compute(PLUS);
	  ASSERT_EQUAL(1, ops.size());
	  ASSERT_EQUAL(0, ops.top());
  }

  void testReq1AdditionNegative(){
	  ops << -10 << 5;
	  calc.compute(PLUS);
	  ASSERT_EQUAL(1, ops.size());
	  ASSERT_EQUAL(-5, ops.top());
  }

  void testReq2Stack() {
	  ops << 6 << 9;
	  calc.compute(PLUS);
      ASSERT_EQUAL(1, ops.size());
	  ASSERT_EQUAL(15, ops.top());

	  ops << 12;
	  calc.compute(PLUS);
      ASSERT_EQUAL(1, ops.size());
	  ASSERT_EQUAL(26, ops.top()); // should be 27 ;-)
  }

  void testReq2StackOutput() {
	  ASSERTM("TODO Req2 Point2", false);
	  //TODO: not yet implemented
  }

  void testReq2StackBlankLine() {
	  ASSERTM("TODO Req2 Point3", false);
	  //TODO: Req2.3 and Req1.1 contradict each other: fix the spec !
  }

  void testReq2Req3AddMult() {
	  ops << 2 << 6 << 9;
	  calc.compute(PLUS);
	  calc.compute(MULTIPLY);
      ASSERT_EQUAL(1, ops.size());
	  ASSERT_EQUAL(30, ops.top());
  }

  void testReq3CommonArtihmentic() {
	  ops << 27;
	  ops << 17;
	  calc.compute(MINUS);
	  ASSERT_EQUAL(10, ops.top());

	  ops << 2;
	  calc.compute(DIVISION);
	  ASSERT_EQUAL(5, ops.top());

	  ops << 8;
	  calc.compute(MULTIPLY);
	  ASSERT_EQUAL(40, ops.top());

	  ops << 2;
	  calc.compute(SQUARE);
	  ASSERT_EQUAL(4, ops.top());

	  ops << 100;
	  calc.compute(NEGATE);
	  ASSERT_EQUAL(-100, ops.top());
  }

  void testReq4UnderflowHandling() {
	  ops.empty();
	  ops << 1;
	  calc.compute(PLUS);
	  ASSERT_EQUAL(0, ops.size());

	  ops.empty();
	  ops << 1;
	  calc.compute(PLUS);
	  ASSERT_EQUAL(0, ops.size());

	  //TODO: Req3.1 and Req3.2 contradict each other: fix the spec !
  }

  void testReq5DirectStackOperations() {
	  ops.empty();
	  ASSERT_EQUAL(0, ops.size());

	  //TODO: Req5.1 to Req5.4 not yet implemented
/*
	  ops.empty();
	  ops << 1;
	  calc.dup();
	  ASSERT_EQUAL(2, ops.size());
	  ASSERT_EQUAL(1, ops.top());
	  ASSERT_EQUAL(1, ops.top());

	  ops.empty();
	  ops << 17;
	  cacl.drop();
	  ASSERT_EQUAL(0, ops.size());

	  ops.empty();
	  ops << 17;
	  ops << 19;
	  cacl.drop();
	  ASSERT_EQUAL(1, ops.size());
	  ASSERT_EQUAL(17, ops.top());

	  ops.empty();
	  ops << 17;
	  cacl.drop();
	  ASSERT_EQUAL(0, ops.size());

	  ops.empty();
	  ops << 17;
	  ops << 19;
	  cacl.drop();
	  ASSERT_EQUAL(0, ops.size());

	  ops.empty();
	  ops << 17;
	  ops << 19;
	  cacl.swap();
	  ASSERT_EQUAL(2, ops.size());
	  ASSERT_EQUAL(17, ops.top());
	  ASSERT_EQUAL(19, ops.top());
*/

  }

  void testReq6SpaceSeparatedEntry() {
	  ops.empty();
	  ASSERT_EQUAL(0, ops.size());

	  ops << 1 << " " << 2;
	  calc.compute(PLUS);
	  ASSERT_EQUAL(3, ops.top());

	  ops << 1 << "\t" << 2;
	  calc.compute(PLUS);
	  ASSERT_EQUAL(3, ops.top());

	  //TODO: Req6.1 are "strings" also valid ???
  }
};


void runSuite(){
	cute::suite s;

	s.push_back(CUTE_SMEMFUN(CalcTest, testReq1Addition));
	s.push_back(CUTE_SMEMFUN(CalcTest, testReq1AdditionZero));
	s.push_back(CUTE_SMEMFUN(CalcTest, testReq1AdditionNegative));

	s.push_back(CUTE_SMEMFUN(CalcTest, testReq2Stack));
//	s.push_back(CUTE_SMEMFUN(CalcTest, testReq2StackOutput));
//	s.push_back(CUTE_SMEMFUN(CalcTest, testReq2StackBlankLine));
	s.push_back(CUTE_SMEMFUN(CalcTest, testReq2Req3AddMult));

	s.push_back(CUTE_SMEMFUN(CalcTest, testReq3CommonArtihmentic));

//	s.push_back(CUTE_SMEMFUN(CalcTest, testReq4UnderflowHandling));

	s.push_back(CUTE_SMEMFUN(CalcTest, testReq5DirectStackOperations));

	s.push_back(CUTE_SMEMFUN(CalcTest, testReq6SpaceSeparatedEntry));

	cute::ide_listener lis;
	cute::makeRunner(lis)(s, "The Suite");
}

int main(){
    runSuite();
}
